@@ -61,8 +61,6 @@ def test_python_cmdline(testcase: DataDrivenTestCase, step: int) -> None:
         args.append("--hide-error-codes")
     if "--disallow-empty-bodies" not in args:
         args.append("--allow-empty-bodies")
-    if "--no-force-uppercase-builtins" not in args:
-        args.append("--force-uppercase-builtins")
     if "--no-force-union-syntax" not in args:
         args.append("--force-union-syntax")
     # Type check the program.
