@@ -30,6 +30,7 @@ from mypy.types import (
     TypeAliasType,
     TypeOfAny,
     TypeType,
+    TypeVarId,
     TypeVarLikeType,
     TypeVarTupleType,
     TypeVarType,
@@ -57,7 +58,7 @@ class TypeFixture:
             return TypeVarType(
                 name,
                 name,
-                id,
+                TypeVarId(id),
                 values,
                 upper_bound,
                 AnyType(TypeOfAny.from_omitted_generics),
@@ -215,6 +216,7 @@ class TypeFixture:
         self.type_d = TypeType.make_normalized(self.d)
         self.type_t = TypeType.make_normalized(self.t)
         self.type_any = TypeType.make_normalized(self.anyt)
+        self.type_never = TypeType.make_normalized(UninhabitedType())
 
         self._add_bool_dunder(self.bool_type_info)
         self._add_bool_dunder(self.ai)
@@ -227,7 +229,7 @@ class TypeFixture:
             return TypeVarTupleType(
                 name,
                 name,
-                id,
+                TypeVarId(id),
                 upper_bound,
                 self.std_tuple,
                 AnyType(TypeOfAny.from_omitted_generics),
@@ -325,7 +327,7 @@ class TypeFixture:
                         TypeVarTupleType(
                             n,
                             n,
-                            id,
+                            TypeVarId(id),
                             self.std_tuple.copy_modified(args=[self.o]),
                             self.std_tuple.copy_modified(args=[self.o]),
                             AnyType(TypeOfAny.from_omitted_generics),
@@ -340,7 +342,7 @@ class TypeFixture:
                         TypeVarType(
                             n,
                             n,
-                            id,
+                            TypeVarId(id),
                             [],
                             self.o,
                             AnyType(TypeOfAny.from_omitted_generics),
