@@ -3,29 +3,110 @@
 Subclass TransformVisitor to perform non-trivial transformations.
 """
 
-from typing import List, Dict, cast, Optional, Iterable
+from __future__ import annotations
+
+from collections.abc import Iterable
+from typing import Optional, cast
 
 from mypy.nodes import (
-    AssertTypeExpr, MypyFile, Import, Node, ImportAll, ImportFrom, FuncItem, FuncDef,
-    OverloadedFuncDef, ClassDef, Decorator, Block, Var,
-    OperatorAssignmentStmt, ExpressionStmt, AssignmentStmt, ReturnStmt,
-    RaiseStmt, AssertStmt, DelStmt, BreakStmt, ContinueStmt,
-    PassStmt, GlobalDecl, WhileStmt, ForStmt, IfStmt, TryStmt, WithStmt,
-    CastExpr, RevealExpr, TupleExpr, GeneratorExpr, ListComprehension, ListExpr,
-    ConditionalExpr, DictExpr, SetExpr, NameExpr, IntExpr, StrExpr, BytesExpr,
-    UnicodeExpr, FloatExpr, CallExpr, SuperExpr, MemberExpr, IndexExpr,
-    SliceExpr, OpExpr, UnaryExpr, LambdaExpr, TypeApplication, PrintStmt,
-    SymbolTable, RefExpr, TypeVarExpr, ParamSpecExpr, NewTypeExpr, PromoteExpr,
-    ComparisonExpr, TempNode, StarExpr, Statement, Expression,
-    YieldFromExpr, NamedTupleExpr, TypedDictExpr, NonlocalDecl, SetComprehension,
-    DictionaryComprehension, ComplexExpr, TypeAliasExpr, EllipsisExpr,
-    YieldExpr, ExecStmt, Argument, BackquoteExpr, AwaitExpr, AssignmentExpr,
-    OverloadPart, EnumCallExpr, REVEAL_TYPE, GDEF, TypeVarTupleExpr
+    GDEF,
+    REVEAL_TYPE,
+    Argument,
+    AssertStmt,
+    AssertTypeExpr,
+    AssignmentExpr,
+    AssignmentStmt,
+    AwaitExpr,
+    Block,
+    BreakStmt,
+    BytesExpr,
+    CallExpr,
+    CastExpr,
+    ClassDef,
+    ComparisonExpr,
+    ComplexExpr,
+    ConditionalExpr,
+    ContinueStmt,
+    Decorator,
+    DelStmt,
+    DictExpr,
+    DictionaryComprehension,
+    EllipsisExpr,
+    EnumCallExpr,
+    Expression,
+    ExpressionStmt,
+    FloatExpr,
+    ForStmt,
+    FuncDef,
+    FuncItem,
+    GeneratorExpr,
+    GlobalDecl,
+    IfStmt,
+    Import,
+    ImportAll,
+    ImportFrom,
+    IndexExpr,
+    IntExpr,
+    LambdaExpr,
+    ListComprehension,
+    ListExpr,
+    MatchStmt,
+    MemberExpr,
+    MypyFile,
+    NamedTupleExpr,
+    NameExpr,
+    NewTypeExpr,
+    Node,
+    NonlocalDecl,
+    OperatorAssignmentStmt,
+    OpExpr,
+    OverloadedFuncDef,
+    OverloadPart,
+    ParamSpecExpr,
+    PassStmt,
+    PromoteExpr,
+    RaiseStmt,
+    RefExpr,
+    ReturnStmt,
+    RevealExpr,
+    SetComprehension,
+    SetExpr,
+    SliceExpr,
+    StarExpr,
+    Statement,
+    StrExpr,
+    SuperExpr,
+    SymbolTable,
+    TempNode,
+    TryStmt,
+    TupleExpr,
+    TypeAliasExpr,
+    TypeApplication,
+    TypedDictExpr,
+    TypeVarExpr,
+    TypeVarTupleExpr,
+    UnaryExpr,
+    Var,
+    WhileStmt,
+    WithStmt,
+    YieldExpr,
+    YieldFromExpr,
+)
+from mypy.patterns import (
+    AsPattern,
+    ClassPattern,
+    MappingPattern,
+    OrPattern,
+    Pattern,
+    SequencePattern,
+    SingletonPattern,
+    StarredPattern,
+    ValuePattern,
 )
-from mypy.types import Type, FunctionLike, ProperType
 from mypy.traverser import TraverserVisitor
-from mypy.visitor import NodeVisitor
+from mypy.types import FunctionLike, ProperType, Type
 from mypy.util import replace_object_state
+from mypy.visitor import NodeVisitor
 
 
 class TransformVisitor(NodeVisitor[Node]):
@@ -55,30 +136,28 @@ class TransformVisitor(NodeVisitor[Node]):
         self.test_only = False
         # There may be multiple references to a Var node. Keep track of
         # Var translations using a dictionary.
-        self.var_map: Dict[Var, Var] = {}
+        self.var_map: dict[Var, Var] = {}
         # These are uninitialized placeholder nodes used temporarily for nested
         # functions while we are transforming a top-level function. This maps an
         # untransformed node to a placeholder (which will later become the
         # transformed node).
-        self.func_placeholder_map: Dict[FuncDef, FuncDef] = {}
+        self.func_placeholder_map: dict[FuncDef, FuncDef] = {}
 
     def visit_mypy_file(self, node: MypyFile) -> MypyFile:
         assert self.test_only, "This visitor should not be used for whole files."
         # NOTE: The 'names' and 'imports' instance variables will be empty!
-        ignored_lines = {line: codes[:]
-                         for line, codes in node.ignored_lines.items()}
-        new = MypyFile(self.statements(node.defs), [], node.is_bom,
-                       ignored_lines=ignored_lines)
+        ignored_lines = {line: codes.copy() for line, codes in node.ignored_lines.items()}
+        new = MypyFile(self.statements(node.defs), [], node.is_bom, ignored_lines=ignored_lines)
         new._fullname = node._fullname
         new.path = node.path
         new.names = SymbolTable()
         return new
 
     def visit_import(self, node: Import) -> Import:
-        return Import(node.ids[:])
+        return Import(node.ids.copy())
 
     def visit_import_from(self, node: ImportFrom) -> ImportFrom:
-        return ImportFrom(node.id, node.relative, node.names[:])
+        return ImportFrom(node.id, node.relative, node.names.copy())
 
     def visit_import_all(self, node: ImportAll) -> ImportAll:
         return ImportAll(node.id, node.relative)
@@ -92,7 +171,7 @@ class TransformVisitor(NodeVisitor[Node]):
         )
 
         # Refresh lines of the inner things
-        arg.set_line(argument.line)
+        arg.set_line(argument)
 
         return arg
 
@@ -110,17 +189,19 @@ class TransformVisitor(NodeVisitor[Node]):
         for stmt in node.body.body:
             stmt.accept(init)
 
-        new = FuncDef(node.name,
-                      [self.copy_argument(arg) for arg in node.arguments],
-                      self.block(node.body),
-                      cast(Optional[FunctionLike], self.optional_type(node.type)))
+        new = FuncDef(
+            node.name,
+            [self.copy_argument(arg) for arg in node.arguments],
+            self.block(node.body),
+            cast(Optional[FunctionLike], self.optional_type(node.type)),
+        )
 
         self.copy_function_attributes(new, node)
 
         new._fullname = node._fullname
         new.is_decorated = node.is_decorated
         new.is_conditional = node.is_conditional
-        new.is_abstract = node.is_abstract
+        new.abstract_status = node.abstract_status
         new.is_static = node.is_static
         new.is_class = node.is_class
         new.is_property = node.is_property
@@ -139,19 +220,23 @@ class TransformVisitor(NodeVisitor[Node]):
             return new
 
     def visit_lambda_expr(self, node: LambdaExpr) -> LambdaExpr:
-        new = LambdaExpr([self.copy_argument(arg) for arg in node.arguments],
-                         self.block(node.body),
-                         cast(Optional[FunctionLike], self.optional_type(node.type)))
+        new = LambdaExpr(
+            [self.copy_argument(arg) for arg in node.arguments],
+            self.block(node.body),
+            cast(Optional[FunctionLike], self.optional_type(node.type)),
+        )
         self.copy_function_attributes(new, node)
         return new
 
-    def copy_function_attributes(self, new: FuncItem,
-                                 original: FuncItem) -> None:
+    def copy_function_attributes(self, new: FuncItem, original: FuncItem) -> None:
         new.info = original.info
         new.min_args = original.min_args
         new.max_pos = original.max_pos
         new.is_overload = original.is_overload
         new.is_generator = original.is_generator
+        new.is_coroutine = original.is_coroutine
+        new.is_async_generator = original.is_async_generator
+        new.is_awaitable_coroutine = original.is_awaitable_coroutine
         new.line = original.line
 
     def visit_overloaded_func_def(self, node: OverloadedFuncDef) -> OverloadedFuncDef:
@@ -173,32 +258,32 @@ class TransformVisitor(NodeVisitor[Node]):
         return new
 
     def visit_class_def(self, node: ClassDef) -> ClassDef:
-        new = ClassDef(node.name,
-                       self.block(node.defs),
-                       node.type_vars,
-                       self.expressions(node.base_type_exprs),
-                       self.optional_expr(node.metaclass))
+        new = ClassDef(
+            node.name,
+            self.block(node.defs),
+            node.type_vars,
+            self.expressions(node.base_type_exprs),
+            self.optional_expr(node.metaclass),
+        )
         new.fullname = node.fullname
         new.info = node.info
-        new.decorators = [self.expr(decorator)
-                          for decorator in node.decorators]
+        new.decorators = [self.expr(decorator) for decorator in node.decorators]
         return new
 
     def visit_global_decl(self, node: GlobalDecl) -> GlobalDecl:
-        return GlobalDecl(node.names[:])
+        return GlobalDecl(node.names.copy())
 
     def visit_nonlocal_decl(self, node: NonlocalDecl) -> NonlocalDecl:
-        return NonlocalDecl(node.names[:])
+        return NonlocalDecl(node.names.copy())
 
     def visit_block(self, node: Block) -> Block:
-        return Block(self.statements(node.body))
+        return Block(self.statements(node.body), is_unreachable=node.is_unreachable)
 
     def visit_decorator(self, node: Decorator) -> Decorator:
         # Note that a Decorator must be transformed to a Decorator.
         func = self.visit_func_def(node.func)
         func.line = node.func.line
-        new = Decorator(func, self.expressions(node.decorators),
-                        self.visit_var(node.var))
+        new = Decorator(func, self.expressions(node.decorators), self.visit_var(node.var))
         new.is_overload = node.is_overload
         return new
 
@@ -220,7 +305,7 @@ class TransformVisitor(NodeVisitor[Node]):
         new.final_value = node.final_value
         new.final_unset_in_class = node.final_unset_in_class
         new.final_set_in_init = node.final_set_in_init
-        new.set_line(node.line)
+        new.set_line(node)
         self.var_map[node] = new
         return new
 
@@ -231,31 +316,34 @@ class TransformVisitor(NodeVisitor[Node]):
         return self.duplicate_assignment(node)
 
     def duplicate_assignment(self, node: AssignmentStmt) -> AssignmentStmt:
-        new = AssignmentStmt(self.expressions(node.lvalues),
-                             self.expr(node.rvalue),
-                             self.optional_type(node.unanalyzed_type))
+        new = AssignmentStmt(
+            self.expressions(node.lvalues),
+            self.expr(node.rvalue),
+            self.optional_type(node.unanalyzed_type),
+        )
         new.line = node.line
         new.is_final_def = node.is_final_def
         new.type = self.optional_type(node.type)
         return new
 
-    def visit_operator_assignment_stmt(self,
-                                       node: OperatorAssignmentStmt) -> OperatorAssignmentStmt:
-        return OperatorAssignmentStmt(node.op,
-                                      self.expr(node.lvalue),
-                                      self.expr(node.rvalue))
+    def visit_operator_assignment_stmt(
+        self, node: OperatorAssignmentStmt
+    ) -> OperatorAssignmentStmt:
+        return OperatorAssignmentStmt(node.op, self.expr(node.lvalue), self.expr(node.rvalue))
 
     def visit_while_stmt(self, node: WhileStmt) -> WhileStmt:
-        return WhileStmt(self.expr(node.expr),
-                         self.block(node.body),
-                         self.optional_block(node.else_body))
+        return WhileStmt(
+            self.expr(node.expr), self.block(node.body), self.optional_block(node.else_body)
+        )
 
     def visit_for_stmt(self, node: ForStmt) -> ForStmt:
-        new = ForStmt(self.expr(node.index),
-                      self.expr(node.expr),
-                      self.block(node.body),
-                      self.optional_block(node.else_body),
-                      self.optional_type(node.unanalyzed_index_type))
+        new = ForStmt(
+            self.expr(node.index),
+            self.expr(node.expr),
+            self.block(node.body),
+            self.optional_block(node.else_body),
+            self.optional_type(node.unanalyzed_index_type),
+        )
         new.is_async = node.is_async
         new.index_type = self.optional_type(node.index_type)
         return new
@@ -270,9 +358,11 @@ class TransformVisitor(NodeVisitor[Node]):
         return DelStmt(self.expr(node.expr))
 
     def visit_if_stmt(self, node: IfStmt) -> IfStmt:
-        return IfStmt(self.expressions(node.expr),
-                      self.blocks(node.body),
-                      self.optional_block(node.else_body))
+        return IfStmt(
+            self.expressions(node.expr),
+            self.blocks(node.body),
+            self.optional_block(node.else_body),
+        )
 
     def visit_break_stmt(self, node: BreakStmt) -> BreakStmt:
         return BreakStmt()
@@ -284,35 +374,76 @@ class TransformVisitor(NodeVisitor[Node]):
         return PassStmt()
 
     def visit_raise_stmt(self, node: RaiseStmt) -> RaiseStmt:
-        return RaiseStmt(self.optional_expr(node.expr),
-                         self.optional_expr(node.from_expr))
+        return RaiseStmt(self.optional_expr(node.expr), self.optional_expr(node.from_expr))
 
     def visit_try_stmt(self, node: TryStmt) -> TryStmt:
-        return TryStmt(self.block(node.body),
-                       self.optional_names(node.vars),
-                       self.optional_expressions(node.types),
-                       self.blocks(node.handlers),
-                       self.optional_block(node.else_body),
-                       self.optional_block(node.finally_body))
+        new = TryStmt(
+            self.block(node.body),
+            self.optional_names(node.vars),
+            self.optional_expressions(node.types),
+            self.blocks(node.handlers),
+            self.optional_block(node.else_body),
+            self.optional_block(node.finally_body),
+        )
+        new.is_star = node.is_star
+        return new
 
     def visit_with_stmt(self, node: WithStmt) -> WithStmt:
-        new = WithStmt(self.expressions(node.expr),
-                       self.optional_expressions(node.target),
-                       self.block(node.body),
-                       self.optional_type(node.unanalyzed_type))
+        new = WithStmt(
+            self.expressions(node.expr),
+            self.optional_expressions(node.target),
+            self.block(node.body),
+            self.optional_type(node.unanalyzed_type),
+        )
         new.is_async = node.is_async
         new.analyzed_types = [self.type(typ) for typ in node.analyzed_types]
         return new
 
-    def visit_print_stmt(self, node: PrintStmt) -> PrintStmt:
-        return PrintStmt(self.expressions(node.args),
-                         node.newline,
-                         self.optional_expr(node.target))
+    def visit_as_pattern(self, p: AsPattern) -> AsPattern:
+        return AsPattern(
+            pattern=self.pattern(p.pattern) if p.pattern is not None else None,
+            name=self.duplicate_name(p.name) if p.name is not None else None,
+        )
 
-    def visit_exec_stmt(self, node: ExecStmt) -> ExecStmt:
-        return ExecStmt(self.expr(node.expr),
-                        self.optional_expr(node.globals),
-                        self.optional_expr(node.locals))
+    def visit_or_pattern(self, p: OrPattern) -> OrPattern:
+        return OrPattern([self.pattern(pat) for pat in p.patterns])
+
+    def visit_value_pattern(self, p: ValuePattern) -> ValuePattern:
+        return ValuePattern(self.expr(p.expr))
+
+    def visit_singleton_pattern(self, p: SingletonPattern) -> SingletonPattern:
+        return SingletonPattern(p.value)
+
+    def visit_sequence_pattern(self, p: SequencePattern) -> SequencePattern:
+        return SequencePattern([self.pattern(pat) for pat in p.patterns])
+
+    def visit_starred_pattern(self, p: StarredPattern) -> StarredPattern:
+        return StarredPattern(self.duplicate_name(p.capture) if p.capture is not None else None)
+
+    def visit_mapping_pattern(self, p: MappingPattern) -> MappingPattern:
+        return MappingPattern(
+            keys=[self.expr(expr) for expr in p.keys],
+            values=[self.pattern(pat) for pat in p.values],
+            rest=self.duplicate_name(p.rest) if p.rest is not None else None,
+        )
+
+    def visit_class_pattern(self, p: ClassPattern) -> ClassPattern:
+        class_ref = p.class_ref.accept(self)
+        assert isinstance(class_ref, RefExpr)
+        return ClassPattern(
+            class_ref=class_ref,
+            positionals=[self.pattern(pat) for pat in p.positionals],
+            keyword_keys=list(p.keyword_keys),
+            keyword_values=[self.pattern(pat) for pat in p.keyword_values],
+        )
+
+    def visit_match_stmt(self, o: MatchStmt) -> MatchStmt:
+        return MatchStmt(
+            subject=self.expr(o.subject),
+            patterns=[self.pattern(p) for p in o.patterns],
+            guards=self.optional_expressions(o.guards),
+            bodies=self.blocks(o.bodies),
+        )
 
     def visit_star_expr(self, node: StarExpr) -> StarExpr:
         return StarExpr(node.expr)
@@ -321,14 +452,11 @@ class TransformVisitor(NodeVisitor[Node]):
         return IntExpr(node.value)
 
     def visit_str_expr(self, node: StrExpr) -> StrExpr:
-        return StrExpr(node.value, node.from_python_3)
+        return StrExpr(node.value)
 
     def visit_bytes_expr(self, node: BytesExpr) -> BytesExpr:
         return BytesExpr(node.value)
 
-    def visit_unicode_expr(self, node: UnicodeExpr) -> UnicodeExpr:
-        return UnicodeExpr(node.value)
-
     def visit_float_expr(self, node: FloatExpr) -> FloatExpr:
         return FloatExpr(node.value)
 
@@ -350,8 +478,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return new
 
     def visit_member_expr(self, node: MemberExpr) -> MemberExpr:
-        member = MemberExpr(self.expr(node.expr),
-                            node.name)
+        member = MemberExpr(self.expr(node.expr), node.name)
         if node.def_var:
             # This refers to an attribute and we don't transform attributes by default,
             # just normal variables.
@@ -387,14 +514,21 @@ class TransformVisitor(NodeVisitor[Node]):
         return AwaitExpr(self.expr(node.expr))
 
     def visit_call_expr(self, node: CallExpr) -> CallExpr:
-        return CallExpr(self.expr(node.callee),
-                        self.expressions(node.args),
-                        node.arg_kinds[:],
-                        node.arg_names[:],
-                        self.optional_expr(node.analyzed))
+        return CallExpr(
+            self.expr(node.callee),
+            self.expressions(node.args),
+            node.arg_kinds.copy(),
+            node.arg_names.copy(),
+            self.optional_expr(node.analyzed),
+        )
 
     def visit_op_expr(self, node: OpExpr) -> OpExpr:
-        new = OpExpr(node.op, self.expr(node.left), self.expr(node.right))
+        new = OpExpr(
+            node.op,
+            self.expr(node.left),
+            self.expr(node.right),
+            cast(Optional[TypeAliasExpr], self.optional_expr(node.analyzed)),
+        )
         new.method_type = self.optional_type(node.method_type)
         return new
 
@@ -404,8 +538,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return new
 
     def visit_cast_expr(self, node: CastExpr) -> CastExpr:
-        return CastExpr(self.expr(node.expr),
-                        self.type(node.type))
+        return CastExpr(self.expr(node.expr), self.type(node.type))
 
     def visit_assert_type_expr(self, node: AssertTypeExpr) -> AssertTypeExpr:
         return AssertTypeExpr(self.expr(node.expr), self.type(node.type))
@@ -426,7 +559,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return new
 
     def visit_assignment_expr(self, node: AssignmentExpr) -> AssignmentExpr:
-        return AssignmentExpr(node.target, node.value)
+        return AssignmentExpr(self.duplicate_name(node.target), self.expr(node.value))
 
     def visit_unary_expr(self, node: UnaryExpr) -> UnaryExpr:
         new = UnaryExpr(node.op, self.expr(node.expr))
@@ -437,8 +570,9 @@ class TransformVisitor(NodeVisitor[Node]):
         return ListExpr(self.expressions(node.items))
 
     def visit_dict_expr(self, node: DictExpr) -> DictExpr:
-        return DictExpr([(self.expr(key) if key else None, self.expr(value))
-                         for key, value in node.items])
+        return DictExpr(
+            [(self.expr(key) if key else None, self.expr(value)) for key, value in node.items]
+        )
 
     def visit_tuple_expr(self, node: TupleExpr) -> TupleExpr:
         return TupleExpr(self.expressions(node.items))
@@ -455,69 +589,85 @@ class TransformVisitor(NodeVisitor[Node]):
                 new.analyzed = self.visit_type_application(node.analyzed)
             else:
                 new.analyzed = self.visit_type_alias_expr(node.analyzed)
-            new.analyzed.set_line(node.analyzed.line)
+            new.analyzed.set_line(node.analyzed)
         return new
 
     def visit_type_application(self, node: TypeApplication) -> TypeApplication:
-        return TypeApplication(self.expr(node.expr),
-                               self.types(node.types))
+        return TypeApplication(self.expr(node.expr), self.types(node.types))
 
     def visit_list_comprehension(self, node: ListComprehension) -> ListComprehension:
         generator = self.duplicate_generator(node.generator)
-        generator.set_line(node.generator.line, node.generator.column)
+        generator.set_line(node.generator)
         return ListComprehension(generator)
 
     def visit_set_comprehension(self, node: SetComprehension) -> SetComprehension:
         generator = self.duplicate_generator(node.generator)
-        generator.set_line(node.generator.line, node.generator.column)
+        generator.set_line(node.generator)
         return SetComprehension(generator)
 
-    def visit_dictionary_comprehension(self, node: DictionaryComprehension
-                                       ) -> DictionaryComprehension:
-        return DictionaryComprehension(self.expr(node.key), self.expr(node.value),
-                                       [self.expr(index) for index in node.indices],
-                                       [self.expr(s) for s in node.sequences],
-                                       [[self.expr(cond) for cond in conditions]
-                                        for conditions in node.condlists],
-                                       node.is_async)
+    def visit_dictionary_comprehension(
+        self, node: DictionaryComprehension
+    ) -> DictionaryComprehension:
+        return DictionaryComprehension(
+            self.expr(node.key),
+            self.expr(node.value),
+            [self.expr(index) for index in node.indices],
+            [self.expr(s) for s in node.sequences],
+            [[self.expr(cond) for cond in conditions] for conditions in node.condlists],
+            node.is_async,
+        )
 
     def visit_generator_expr(self, node: GeneratorExpr) -> GeneratorExpr:
         return self.duplicate_generator(node)
 
     def duplicate_generator(self, node: GeneratorExpr) -> GeneratorExpr:
-        return GeneratorExpr(self.expr(node.left_expr),
-                             [self.expr(index) for index in node.indices],
-                             [self.expr(s) for s in node.sequences],
-                             [[self.expr(cond) for cond in conditions]
-                              for conditions in node.condlists],
-                             node.is_async)
+        return GeneratorExpr(
+            self.expr(node.left_expr),
+            [self.expr(index) for index in node.indices],
+            [self.expr(s) for s in node.sequences],
+            [[self.expr(cond) for cond in conditions] for conditions in node.condlists],
+            node.is_async,
+        )
 
     def visit_slice_expr(self, node: SliceExpr) -> SliceExpr:
-        return SliceExpr(self.optional_expr(node.begin_index),
-                         self.optional_expr(node.end_index),
-                         self.optional_expr(node.stride))
+        return SliceExpr(
+            self.optional_expr(node.begin_index),
+            self.optional_expr(node.end_index),
+            self.optional_expr(node.stride),
+        )
 
     def visit_conditional_expr(self, node: ConditionalExpr) -> ConditionalExpr:
-        return ConditionalExpr(self.expr(node.cond),
-                               self.expr(node.if_expr),
-                               self.expr(node.else_expr))
-
-    def visit_backquote_expr(self, node: BackquoteExpr) -> BackquoteExpr:
-        return BackquoteExpr(self.expr(node.expr))
+        return ConditionalExpr(
+            self.expr(node.cond), self.expr(node.if_expr), self.expr(node.else_expr)
+        )
 
     def visit_type_var_expr(self, node: TypeVarExpr) -> TypeVarExpr:
-        return TypeVarExpr(node.name, node.fullname,
-                           self.types(node.values),
-                           self.type(node.upper_bound), variance=node.variance)
+        return TypeVarExpr(
+            node.name,
+            node.fullname,
+            self.types(node.values),
+            self.type(node.upper_bound),
+            self.type(node.default),
+            variance=node.variance,
+        )
 
     def visit_paramspec_expr(self, node: ParamSpecExpr) -> ParamSpecExpr:
         return ParamSpecExpr(
-            node.name, node.fullname, self.type(node.upper_bound), variance=node.variance
+            node.name,
+            node.fullname,
+            self.type(node.upper_bound),
+            self.type(node.default),
+            variance=node.variance,
         )
 
     def visit_type_var_tuple_expr(self, node: TypeVarTupleExpr) -> TypeVarTupleExpr:
         return TypeVarTupleExpr(
-            node.name, node.fullname, self.type(node.upper_bound), variance=node.variance
+            node.name,
+            node.fullname,
+            self.type(node.upper_bound),
+            node.tuple_fallback,
+            self.type(node.default),
+            variance=node.variance,
         )
 
     def visit_type_alias_expr(self, node: TypeAliasExpr) -> TypeAliasExpr:
@@ -545,32 +695,38 @@ class TransformVisitor(NodeVisitor[Node]):
 
     def node(self, node: Node) -> Node:
         new = node.accept(self)
-        new.set_line(node.line)
+        new.set_line(node)
         return new
 
     def mypyfile(self, node: MypyFile) -> MypyFile:
         new = node.accept(self)
         assert isinstance(new, MypyFile)
-        new.set_line(node.line)
+        new.set_line(node)
         return new
 
     def expr(self, expr: Expression) -> Expression:
         new = expr.accept(self)
         assert isinstance(new, Expression)
-        new.set_line(expr.line, expr.column)
+        new.set_line(expr)
         return new
 
     def stmt(self, stmt: Statement) -> Statement:
         new = stmt.accept(self)
         assert isinstance(new, Statement)
-        new.set_line(stmt.line, stmt.column)
+        new.set_line(stmt)
+        return new
+
+    def pattern(self, pattern: Pattern) -> Pattern:
+        new = pattern.accept(self)
+        assert isinstance(new, Pattern)
+        new.set_line(pattern)
         return new
 
     # Helpers
     #
     # All the node helpers also propagate line numbers.
 
-    def optional_expr(self, expr: Optional[Expression]) -> Optional[Expression]:
+    def optional_expr(self, expr: Expression | None) -> Expression | None:
         if expr:
             return self.expr(expr)
         else:
@@ -581,30 +737,31 @@ class TransformVisitor(NodeVisitor[Node]):
         new.line = block.line
         return new
 
-    def optional_block(self, block: Optional[Block]) -> Optional[Block]:
+    def optional_block(self, block: Block | None) -> Block | None:
         if block:
             return self.block(block)
         else:
             return None
 
-    def statements(self, statements: List[Statement]) -> List[Statement]:
+    def statements(self, statements: list[Statement]) -> list[Statement]:
         return [self.stmt(stmt) for stmt in statements]
 
-    def expressions(self, expressions: List[Expression]) -> List[Expression]:
+    def expressions(self, expressions: list[Expression]) -> list[Expression]:
         return [self.expr(expr) for expr in expressions]
 
-    def optional_expressions(self, expressions: Iterable[Optional[Expression]]
-                             ) -> List[Optional[Expression]]:
+    def optional_expressions(
+        self, expressions: Iterable[Expression | None]
+    ) -> list[Expression | None]:
         return [self.optional_expr(expr) for expr in expressions]
 
-    def blocks(self, blocks: List[Block]) -> List[Block]:
+    def blocks(self, blocks: list[Block]) -> list[Block]:
         return [self.block(block) for block in blocks]
 
-    def names(self, names: List[NameExpr]) -> List[NameExpr]:
+    def names(self, names: list[NameExpr]) -> list[NameExpr]:
         return [self.duplicate_name(name) for name in names]
 
-    def optional_names(self, names: Iterable[Optional[NameExpr]]) -> List[Optional[NameExpr]]:
-        result: List[Optional[NameExpr]] = []
+    def optional_names(self, names: Iterable[NameExpr | None]) -> list[NameExpr | None]:
+        result: list[NameExpr | None] = []
         for name in names:
             if name:
                 result.append(self.duplicate_name(name))
@@ -616,13 +773,13 @@ class TransformVisitor(NodeVisitor[Node]):
         # Override this method to transform types.
         return type
 
-    def optional_type(self, type: Optional[Type]) -> Optional[Type]:
+    def optional_type(self, type: Type | None) -> Type | None:
         if type:
             return self.type(type)
         else:
             return None
 
-    def types(self, types: List[Type]) -> List[Type]:
+    def types(self, types: list[Type]) -> list[Type]:
         return [self.type(type) for type in types]
 
 
@@ -639,5 +796,6 @@ class FuncMapInitializer(TraverserVisitor):
         if node not in self.transformer.func_placeholder_map:
             # Haven't seen this FuncDef before, so create a placeholder node.
             self.transformer.func_placeholder_map[node] = FuncDef(
-                node.name, node.arguments, node.body, None)
+                node.name, node.arguments, node.body, None
+            )
         super().visit_func_def(node)
