@@ -31,14 +31,7 @@ from mypyc.ir.rtypes import (
     str_rprimitive,
     void_rtype,
 )
-from mypyc.primitives.registry import (
-    CFunctionDescription,
-    binary_op,
-    custom_op,
-    function_op,
-    load_address_op,
-    unary_op,
-)
+from mypyc.primitives.registry import binary_op, custom_op, function_op, load_address_op, unary_op
 
 # Constructors for builtins.int and native int types have the same behavior. In
 # interpreted mode, native int types are just aliases to 'int'.
@@ -84,25 +77,25 @@ for int_name in (
         error_kind=ERR_MAGIC,
     )
 
-# str(int)
-int_to_str_op = function_op(
-    name="builtins.str",
-    arg_types=[int_rprimitive],
-    return_type=str_rprimitive,
-    c_function_name="CPyTagged_Str",
-    error_kind=ERR_MAGIC,
-    priority=2,
-)
-
-# We need a specialization for str on bools also since the int one is wrong...
-function_op(
-    name="builtins.str",
-    arg_types=[bool_rprimitive],
-    return_type=str_rprimitive,
-    c_function_name="CPyBool_Str",
-    error_kind=ERR_MAGIC,
-    priority=3,
-)
+for name in ("builtins.str", "builtins.repr"):
+    # str(int) and repr(int)
+    int_to_str_op = function_op(
+        name=name,
+        arg_types=[int_rprimitive],
+        return_type=str_rprimitive,
+        c_function_name="CPyTagged_Str",
+        error_kind=ERR_MAGIC,
+        priority=2,
+    )
+    # We need a specialization for str on bools also since the int one is wrong...
+    function_op(
+        name=name,
+        arg_types=[bool_rprimitive],
+        return_type=str_rprimitive,
+        c_function_name="CPyBool_Str",
+        error_kind=ERR_MAGIC,
+        priority=3,
+    )
 
 
 def int_binary_primitive(
@@ -176,7 +169,7 @@ int_binary_op(">>=", "CPyTagged_Rshift", error_kind=ERR_MAGIC)
 int_binary_op("<<=", "CPyTagged_Lshift", error_kind=ERR_MAGIC)
 
 
-def int_unary_op(name: str, c_function_name: str) -> CFunctionDescription:
+def int_unary_op(name: str, c_function_name: str) -> PrimitiveDescription:
     return unary_op(
         name=name,
         arg_type=int_rprimitive,
@@ -199,6 +192,7 @@ int_equal_ = custom_op(
     return_type=bit_rprimitive,
     c_function_name="CPyTagged_IsEq_",
     error_kind=ERR_NEVER,
+    is_pure=True,
 )
 
 # Less than operation on two boxed tagged integers
@@ -207,6 +201,7 @@ int_less_than_ = custom_op(
     return_type=bit_rprimitive,
     c_function_name="CPyTagged_IsLt_",
     error_kind=ERR_NEVER,
+    is_pure=True,
 )
 
 int64_divide_op = custom_op(
